"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");
exports.__esModule = true;
exports.default = void 0;
var _apidomCore = require("@swagger-api/apidom-core");
var _apidomJsonPointer = require("@swagger-api/apidom-json-pointer");
var _ramda = require("ramda");
var _index = require("./errors/index.cjs");
var _parse = _interopRequireDefault(require("./parse.cjs"));
// evaluates Relative JSON Pointer against ApiDOM fragment
const evaluate = (relativePointer, currentElement, rootElement) => {
  let ancestorLineage = [];
  let cursor = currentElement;
  (0, _apidomCore.visit)(rootElement, {
    enter(element, key, parent, path, ancestors) {
      if (element === currentElement) {
        ancestorLineage = [...ancestors, parent].filter(_apidomCore.isElement);
        return _apidomCore.BREAK;
      }
      return undefined;
    }
  });
  if (ancestorLineage.length === 0) {
    throw new _index.EvaluationRelativeJsonPointerError('Current element not found inside the root element');
  }
  if ((0, _ramda.last)(ancestorLineage) === rootElement) {
    throw new _index.EvaluationRelativeJsonPointerError('Current element cannot be the root element');
  }
  const relativeJsonPointer = (0, _parse.default)(relativePointer);

  // non-negative-integer
  if (relativeJsonPointer.nonNegativeIntegerPrefix > 0) {
    const ancestorLineageCopy = [...ancestorLineage];
    for (let {
      nonNegativeIntegerPrefix
    } = relativeJsonPointer; nonNegativeIntegerPrefix > 0; nonNegativeIntegerPrefix -= 1) {
      cursor = ancestorLineageCopy.pop();
      if ((0, _apidomCore.isMemberElement)(cursor)) {
        cursor = ancestorLineageCopy.pop();
      }
    }
    if (typeof cursor === 'undefined') {
      throw new _index.EvaluationRelativeJsonPointerError(`Evaluation failed on non-negative-integer prefix of "${relativeJsonPointer.nonNegativeIntegerPrefix}"`);
    }
    ancestorLineage = ancestorLineageCopy;
  }

  // index-manipulation
  if (typeof relativeJsonPointer.indexManipulation === 'number') {
    const containedArray = (0, _ramda.last)(ancestorLineage);
    if (typeof containedArray === 'undefined' || !(0, _apidomCore.isArrayElement)(containedArray)) {
      throw new _index.EvaluationRelativeJsonPointerError(`Evaluation failed on index-manipulation "${relativeJsonPointer.indexManipulation}"`);
    }
    const currentCursorIndex = containedArray.content.indexOf(cursor);
    const newCursorIndex = currentCursorIndex + relativeJsonPointer.indexManipulation;
    cursor = containedArray.content[newCursorIndex];
    if (typeof cursor === 'undefined') {
      throw new _index.EvaluationRelativeJsonPointerError(`Evaluation failed on index-manipulation "${relativeJsonPointer.indexManipulation}"`);
    }
  }
  if (Array.isArray(relativeJsonPointer.jsonPointerTokens)) {
    // <json-pointer>
    const jsonPointer = (0, _apidomJsonPointer.compile)(relativeJsonPointer.jsonPointerTokens);
    cursor = (0, _apidomJsonPointer.evaluate)(jsonPointer, cursor);
  } else if (relativeJsonPointer.hashCharacter) {
    // "#"
    if (cursor === rootElement) {
      throw new _index.EvaluationRelativeJsonPointerError('Current element cannot be the root element to apply "#"');
    }
    const parentElement = (0, _ramda.last)(ancestorLineage);
    if (typeof parentElement !== 'undefined') {
      if ((0, _apidomCore.isMemberElement)(parentElement)) {
        cursor = parentElement.key;
      } else if ((0, _apidomCore.isArrayElement)(parentElement)) {
        cursor = new _apidomCore.NumberElement(parentElement.content.indexOf(cursor));
      }
    }
  }
  return cursor;
};
var _default = evaluate;
exports.default = _default;