"use strict";

exports.__esModule = true;
exports.isRelativeJsonPointer = exports.default = void 0;
var _apidomJsonPointer = require("@swagger-api/apidom-json-pointer");
var _index = require("./errors/index.cjs");
const nonNegativeIntegerPrefixRegExp = '(?<nonNegativeIntegerPrefix>[1-9]\\d*|0)';
const indexManipulationRegExp = '(?<indexManipulation>[+-][1-9]\\d*|0)';
const hashCharacterRegExp = '(?<hashCharacter>#)';
const jsonPointerRegExp = '(?<jsonPointer>\\/.*)';
const relativeJsonPointerRegExp = new RegExp(`^${nonNegativeIntegerPrefixRegExp}${indexManipulationRegExp}?(${hashCharacterRegExp}|${jsonPointerRegExp})?$`);
const isRelativeJsonPointer = value => {
  return typeof value === 'string' && relativeJsonPointerRegExp.test(value);
};

// parse :: String -> RelativeJsonPointer
exports.isRelativeJsonPointer = isRelativeJsonPointer;
const parse = relativePointer => {
  const match = relativePointer.match(relativeJsonPointerRegExp);
  if (match === null || typeof match.groups === 'undefined') {
    throw new _index.InvalidRelativeJsonPointerError(relativePointer);
  }

  // non-negative-integer
  const nonNegativeIntegerPrefix = parseInt(match.groups.nonNegativeIntegerPrefix, 10);
  // index-manipulation
  const indexManipulation = typeof match.groups.indexManipulation === 'string' ? parseInt(match.groups.indexManipulation, 10) : undefined;
  // <json-pointer>
  const jsonPointerTokens = typeof match.groups.jsonPointer === 'string' ? (0, _apidomJsonPointer.parse)(match.groups.jsonPointer) : undefined;
  // "#"
  const hashCharacter = typeof match.groups.hashCharacter === 'string';
  return {
    nonNegativeIntegerPrefix,
    indexManipulation,
    jsonPointerTokens,
    hashCharacter
  };
};
var _default = parse;
exports.default = _default;